#version 330 core

struct SimpleDirectionalLight {
	vec4 vColor;
	vec3 vDirection;
	float fAmbientIntensity;
};

struct Material {
	vec4 emission;
	vec4 ambient;
	vec4 diffuse;
	vec4 specular;
	float shininess;
};

uniform SimpleDirectionalLight sunLight;
uniform vec3 materialColor;
uniform sampler2D textureSample;
uniform Material material;
smooth in vec3 vNormal;
in vec2 uv;
in vec4 eye;

out vec4 outputColor;


void main(){
	vec3 diffuseColor = max(dot(sunLight.vDirection, normalize(vNormal)),0.0)*sunLight.vColor*material.diffuse;
	vec3 reflection = normalize(reflect(-sunDirection, normalize(vNormal)));
	float direction = max(0.0, dot(normalize(eye.xyz), reflection));
	vec3 specular = pow(direction, material.shininess)*sunLight.vColor*material.spec;
	float intensity = max(0.0, dot(normalize(vNormal), -sunLight.vDirection));
	float specTmp = 0;
	float specularIntensity = 0;
	if (dot(-sunLight.vDirection, vNormal) > 0.0) {
		vec3 halfwayVector = normalize(-sunLight.vDirection + eye.xyz);
		specTmp = max(dot(vNormal, halfwayVector), 0.0);
		specularIntensity = pow(specTmp, material.shininess);
	}
	intensity = intensity + specularIntensity;
	outputColor = texture(textureSample, uv) * max(intensity * material.diffuse , material.ambient);
	outputColor.a = texture(textureSample, uv).a;
    if(outputColor.a == 0.0){
        discard;
    }
}
