#version 330 core

precision highp float;

in vec2 textureCoordinate;
in vec2 leftTextureCoordinate;
in vec2 rightTextureCoordinate;

in vec2 topTextureCoordinate;
in vec2 topLeftTextureCoordinate;
in vec2 topRightTextureCoordinate;

in vec2 bottomTextureCoordinate;
in vec2 bottomLeftTextureCoordinate;
in vec2 bottomRightTextureCoordinate;

// texture sampler
uniform sampler2D color;
uniform sampler2D normals;

// output fragment colour RGBA
out vec4 outputColor;

float mapIntensity(float intensity) {
	if (intensity > 0.99f) return 1.0f;
	else if (intensity > 0.9f) return 0.9f;
	else if (intensity > 0.8f) return 0.8f;
	else if (intensity > 0.7f) return 0.7f;
	else if (intensity > 0.6f) return 0.6f;
	else if (intensity > 0.5f) return 0.5f;
	else if (intensity > 0.4f) return 0.4f;
	else if (intensity > 0.3f) return 0.3f;
	else return 0.0f;
}

void main () {
	float i00   = (0.2126 * texture2D(normals, textureCoordinate).r + 0.7152 * texture2D(normals, textureCoordinate).g + 0.0722 * texture2D(normals, textureCoordinate).b);
	float im1m1 = (0.2126 * texture2D(normals, bottomLeftTextureCoordinate).r + 0.7152 * texture2D(normals, bottomLeftTextureCoordinate).g + 0.0722 * texture2D(normals, bottomLeftTextureCoordinate).b);
	float ip1p1 = (0.2126 * texture2D(normals, topRightTextureCoordinate).r + 0.7152 * texture2D(normals, topRightTextureCoordinate).g + 0.0722 * texture2D(normals, topRightTextureCoordinate).b);
	float im1p1 = (0.2126 * texture2D(normals, topLeftTextureCoordinate).r + 0.7152 * texture2D(normals, topLeftTextureCoordinate).g + 0.0722 * texture2D(normals, topLeftTextureCoordinate).b);
	float ip1m1 = (0.2126 * texture2D(normals, bottomRightTextureCoordinate).r + 0.7152 * texture2D(normals, bottomRightTextureCoordinate).g + 0.0722 * texture2D(normals, bottomRightTextureCoordinate).b);
	float im10 = (0.2126 * texture2D(normals, leftTextureCoordinate).r + 0.7152 * texture2D(normals, leftTextureCoordinate).g + 0.0722 * texture2D(normals, leftTextureCoordinate).b);
	float ip10 = (0.2126 * texture2D(normals, rightTextureCoordinate).r + 0.7152 * texture2D(normals, rightTextureCoordinate).g + 0.0722 * texture2D(normals, rightTextureCoordinate).b);
	float i0m1 = (0.2126 * texture2D(normals, bottomTextureCoordinate).r + 0.7152 * texture2D(normals, bottomTextureCoordinate).g + 0.0722 * texture2D(normals, bottomTextureCoordinate).b);
	float i0p1 = (0.2126 * texture2D(normals, topTextureCoordinate).r + 0.7152 * texture2D(normals, topTextureCoordinate).g + 0.0722 * texture2D(normals, topTextureCoordinate).b);
	float h = -im1p1 - 2.0f * i0p1 - ip1p1 + im1m1 + 2.0f * i0m1 + ip1m1;
	float v = -im1m1 - 2.0f * im10 - im1p1 + ip1m1 + 2.0f * ip10 + ip1p1;

	float mag = length(vec2(h, v));

	if (mag < 0.3f) {
		outputColor = texture(color, textureCoordinate);
	} else {
		outputColor = vec4(vec3(0.f), 1.f);
	}
	outputColor = vec4(texture(color,textureCoordinate).rgb - mapIntensity(mag), 1.f);
}
